package function.builtin;

import function.ArgumentValidator.BadArgumentTypeException;
import function.ArgumentValidator.TooFewArgumentsException;
import function.ArgumentValidator.TooManyArgumentsException;
import function.builtin.EVAL.UndefinedSymbolException;
import org.junit.Test;
import sexpression.LispNumber;
import table.SymbolTable;
import testutil.SymbolAndFunctionCleaner;

import static org.junit.Assert.assertNull;
import static testutil.TestUtilities.assertSExpressionsMatch;
import static testutil.TestUtilities.evaluateString;

public class SETTest extends SymbolAndFunctionCleaner {

    @Test
    public void set() {
        evaluateString("(set 'a 23)");
        assertSExpressionsMatch(new LispNumber("23"), evaluateString("a"));
    }

    @Test
    public void lookupDefinedSymbol() {
        evaluateString("(set 'a 23)");
        assertSExpressionsMatch(new LispNumber("23"), getExecutionContext().lookupSymbolValue("A"));
    }

    @Test
    public void lookupUndefinedSymbol() {
        assertNull(getExecutionContext().lookupSymbolValue("A"));
    }

    @Test
    public void setGlobalVariable() {
        evaluateString("(set 'a 23)");
        SymbolTable global = getExecutionContext().getScope();
        getExecutionContext().setScope(new SymbolTable(global));

        evaluateString("(set 'a 94)");
        getExecutionContext().setScope(global);
        assertSExpressionsMatch(new LispNumber("94"), evaluateString("a"));
    }

    @Test(expected = UndefinedSymbolException.class)
    public void setLocalVariableDefined_DoesNotSetGlobal() {
        SymbolTable global = getExecutionContext().getScope();
        SymbolTable local = new SymbolTable(global);
        local.set("A", new LispNumber("99"));
        getExecutionContext().setScope(local);

        evaluateString("(set 'a 94)");
        getExecutionContext().setScope(global);
        evaluateString("a");
    }

    @Test
    public void setLocalVariableUndefined_SetsGlobal() {
        SymbolTable global = getExecutionContext().getScope();
        SymbolTable local = new SymbolTable(global);
        getExecutionContext().setScope(local);

        evaluateString("(set 'a 94)");
        getExecutionContext().setScope(global);
        assertSExpressionsMatch(new LispNumber("94"), evaluateString("a"));
    }

    @Test(expected = BadArgumentTypeException.class)
    public void setWithNonSymbol() {
        evaluateString("(set '1 2)");
    }

    @Test(expected = TooFewArgumentsException.class)
    public void setWithTooFewArguments() {
        evaluateString("(set 'x)");
    }

    @Test(expected = TooManyArgumentsException.class)
    public void setWithTooManyArguments() {
        evaluateString("(set 'a 'b 'c)");
    }
}
